@@ -237,6 +237,11 @@ static struct GNUNET_DB_EventHandler *eh;
  */
 static unsigned int active_inquiries;
 
+/**
+ * Set to true if we ever encountered any problem.
+ */
+static bool found_problem;
+
 /**
  * Value to return from main(). 0 on success, non-zero on errors.
  */
@@ -394,6 +399,8 @@ update_transaction_status (const struct Inquiry *w,
 {
   enum GNUNET_DB_QueryStatus qs;
 
+  if (failed)
+    found_problem = true;
   qs = db_plugin->update_transfer_status (db_plugin->cls,
                                           w->exchange->exchange_url,
                                           &w->wtid,
@@ -476,6 +483,7 @@ end_inquiry (struct Inquiry *w)
        (at_limit) )
   {
     at_limit = false;
+    GNUNET_assert (NULL == task);
     task = GNUNET_SCHEDULER_add_now (&find_work,
                                      NULL);
   }
@@ -539,6 +547,11 @@ shutdown_task (void *cls)
     db_plugin->event_listen_cancel (eh);
     eh = NULL;
   }
+  if (NULL != task)
+  {
+    GNUNET_SCHEDULER_cancel (task);
+    task = NULL;
+  }
   TALER_MERCHANTDB_plugin_unload (db_plugin);
   db_plugin = NULL;
   cfg = NULL;
@@ -707,12 +720,32 @@ check_transfer (void *cls,
     GNUNET_break (0);
     return;   /* already had a serious issue; odd that we're called more than once as well... */
   }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Checking coin with value %s\n",
+              TALER_amount2s (amount_with_fee));
   if ( (GNUNET_OK !=
         TALER_amount_cmp_currency (amount_with_fee,
                                    &ttd->coin_value)) ||
        (0 != TALER_amount_cmp (amount_with_fee,
-                               &ttd->coin_value)) ||
-       (GNUNET_OK !=
+                               &ttd->coin_value)) )
+  {
+    /* Disagreement between the exchange and us about how much this
+       coin is worth! */
+    GNUNET_break_op (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Disagreement about coin value %s\n",
+                TALER_amount2s (amount_with_fee));
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Exchange gave it a value of %s\n",
+                TALER_amount2s (&ttd->coin_value));
+    ctc->check_transfer_result = GNUNET_SYSERR;
+    /* Build the `TrackTransferConflictDetails` */
+    ctc->ec = TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_CONFLICTING_REPORTS;
+    ctc->failure = true;
+    /* FIXME: this should be reported to the auditor (once the auditor has an API for this) */
+    return;
+  }
+  if ( (GNUNET_OK !=
         TALER_amount_cmp_currency (deposit_fee,
                                    &ttd->coin_fee)) ||
        (0 != TALER_amount_cmp (deposit_fee,
@@ -721,11 +754,17 @@ check_transfer (void *cls,
     /* Disagreement between the exchange and us about how much this
        coin is worth! */
     GNUNET_break_op (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Expected fee is %s\n",
+                TALER_amount2s (&ttd->coin_fee));
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Fee claimed by exchange is %s\n",
+                TALER_amount2s (deposit_fee));
     ctc->check_transfer_result = GNUNET_SYSERR;
     /* Build the `TrackTransferConflictDetails` */
     ctc->ec = TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_CONFLICTING_REPORTS;
     ctc->failure = true;
-    /* TODO: this should be reported to the auditor! */
+    /* FIXME: this should be reported to the auditor (once the auditor has an API for this) */
     return;
   }
   ctc->check_transfer_result = GNUNET_OK;
@@ -928,6 +967,12 @@ wire_transfer_cb (void *cls,
                           &w->total)) )
   {
     GNUNET_break_op (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Wire transfer total value was %s\n",
+                TALER_amount2s (&w->total));
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Exchange claimed total value to be %s\n",
+                TALER_amount2s (&td->total_amount));
     update_transaction_status (w,
                                GNUNET_TIME_UNIT_FOREVER_ABS,
                                TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_CONFLICTING_TRANSFERS,
@@ -1203,6 +1248,7 @@ run (void *cls,
                                   &transfer_added,
                                   NULL);
   }
+  GNUNET_assert (NULL == task);
   task = GNUNET_SCHEDULER_add_now (&find_work,
                                    NULL);
 }
@@ -1248,6 +1294,9 @@ main (int argc,
     return EXIT_INVALIDARGUMENT;
   if (GNUNET_NO == ret)
     return EXIT_SUCCESS;
+  if ( (found_problem) &&
+       (0 == global_ret) )
+    global_ret = 7;
   return global_ret;
 }
 
