@@ -418,7 +418,7 @@ struct PayContext
    * Number of coins this payment is made of.  Length
    * of the @e dc array.
    */
-  unsigned int coins_cnt;
+  size_t coins_cnt;
 
   /**
    * Number of exchanges involved in the payment. Length
@@ -476,69 +476,6 @@ struct PayContext
 };
 
 
-/**
- * Active KYC operation with an exchange.
- */
-struct KycContext
-{
-  /**
-   * Kept in a DLL.
-   */
-  struct KycContext *next;
-
-  /**
-   * Kept in a DLL.
-   */
-  struct KycContext *prev;
-
-  /**
-   * Looking for the exchange.
-   */
-  struct TMH_EXCHANGES_KeysOperation *fo;
-
-  /**
-   * Exchange this is about.
-   */
-  char *exchange_url;
-
-  /**
-   * Merchant instance this is for.
-   */
-  struct TMH_MerchantInstance *mi;
-
-  /**
-   * Wire method we are checking the status of.
-   */
-  struct TMH_WireMethod *wm;
-
-  /**
-   * Handle for the GET /deposits operation.
-   */
-  struct TALER_EXCHANGE_DepositGetHandle *dg;
-
-  /**
-   * Contract we are looking up.
-   */
-  struct TALER_PrivateContractHashP h_contract_terms;
-
-  /**
-   * Coin we are looking up.
-   */
-  struct TALER_CoinSpendPublicKeyP coin_pub;
-
-  /**
-   * Initial DB timestamp.
-   */
-  struct GNUNET_TIME_Timestamp kyc_timestamp;
-
-  /**
-   * Initial KYC status.
-   */
-  bool kyc_ok;
-
-};
-
-
 /**
  * Head of active pay context DLL.
  */
@@ -549,57 +486,10 @@ static struct PayContext *pc_head;
  */
 static struct PayContext *pc_tail;
 
-/**
- * Head of active KYC context DLL.
- */
-static struct KycContext *kc_head;
-
-/**
- * Tail of active KYC context DLL.
- */
-static struct KycContext *kc_tail;
-
-
-/**
- * Free resources used by @a kc.
- *
- * @param[in] kc object to free
- */
-static void
-destroy_kc (struct KycContext *kc)
-{
-  if (NULL != kc->fo)
-  {
-    TMH_EXCHANGES_keys4exchange_cancel (kc->fo);
-    kc->fo = NULL;
-  }
-  if (NULL != kc->dg)
-  {
-    TALER_EXCHANGE_deposits_get_cancel (kc->dg);
-    kc->dg = NULL;
-  }
-  TMH_instance_decref (kc->mi);
-  kc->mi = NULL;
-  GNUNET_free (kc->exchange_url);
-  GNUNET_CONTAINER_DLL_remove (kc_head,
-                               kc_tail,
-                               kc);
-  GNUNET_free (kc);
-}
-
 
 void
 TMH_force_pc_resume ()
 {
-  struct KycContext *kc;
-
-  while (NULL != (kc = kc_head))
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Aborting KYC check at %s\n",
-                kc->exchange_url);
-    destroy_kc (kc);
-  }
   for (struct PayContext *pc = pc_head;
        NULL != pc;
        pc = pc->next)
@@ -652,7 +542,6 @@ resume_pay_with_response (struct PayContext *pc,
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Resuming /pay handling. HTTP status for our reply is %u.\n",
               response_code);
-#if 1
   for (unsigned int i = 0; i<pc->num_exchanges; i++)
   {
     struct ExchangeGroup *eg = pc->egs[i];
@@ -667,10 +556,10 @@ resume_pay_with_response (struct PayContext *pc,
     {
       TALER_EXCHANGE_batch_deposit_cancel (eg->bdh);
       eg->bdh = NULL;
+      pc->pending_at_eg--;
     }
   }
   GNUNET_assert (0 == pc->pending_at_eg);
-#endif
   if (NULL != pc->timeout_task)
   {
     GNUNET_SCHEDULER_cancel (pc->timeout_task);
@@ -742,221 +631,6 @@ phase_return_response (struct PayContext *pc)
 }
 
 
-/**
- * Function called with detailed wire transfer data.
- *
- * @param cls a `struct KycContext *`
- * @param dr HTTP response data
- */
-static void
-deposit_get_callback (
-  void *cls,
-  const struct TALER_EXCHANGE_GetDepositResponse *dr)
-{
-  struct KycContext *kc = cls;
-  enum GNUNET_DB_QueryStatus qs;
-  struct GNUNET_TIME_Timestamp now;
-
-  kc->dg = NULL;
-  now = GNUNET_TIME_timestamp_get ();
-  switch (dr->hr.http_status)
-  {
-  case MHD_HTTP_OK:
-    qs = TMH_db->account_kyc_set_status (
-      TMH_db->cls,
-      kc->mi->settings.id,
-      &kc->wm->h_wire,
-      kc->exchange_url,
-      0LL,
-      NULL, /* no signature */
-      NULL, /* no signature */
-      now,
-      true,
-      TALER_AML_NORMAL);
-    GNUNET_break (qs > 0);
-    break;
-  case MHD_HTTP_ACCEPTED:
-    qs = TMH_db->account_kyc_set_status (
-      TMH_db->cls,
-      kc->mi->settings.id,
-      &kc->wm->h_wire,
-      kc->exchange_url,
-      dr->details.accepted.requirement_row,
-      NULL, /* no signature */
-      NULL, /* no signature */
-      now,
-      dr->details.accepted.kyc_ok,
-      dr->details.accepted.aml_decision);
-    GNUNET_break (qs > 0);
-    break;
-  default:
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "KYC check failed at %s with unexpected status %u\n",
-                kc->exchange_url,
-                dr->hr.http_status);
-  }
-  destroy_kc (kc);
-}
-
-
-/**
- * Function called with the result of our exchange lookup.
- *
- * @param cls the `struct KycContext`
- * @param keys NULL if exchange was not found to be acceptable
- * @param exchange representation of the exchange
- */
-static void
-process_kyc_with_exchange (
-  void *cls,
-  struct TALER_EXCHANGE_Keys *keys,
-  struct TMH_Exchange *exchange)
-{
-  struct KycContext *kc = cls;
-
-  (void) exchange;
-  kc->fo = NULL;
-  if (NULL == keys)
-  {
-    destroy_kc (kc);
-    return;
-  }
-  kc->dg = TALER_EXCHANGE_deposits_get (
-    TMH_curl_ctx,
-    kc->exchange_url,
-    keys,
-    &kc->mi->merchant_priv,
-    &kc->wm->h_wire,
-    &kc->h_contract_terms,
-    &kc->coin_pub,
-    GNUNET_TIME_UNIT_ZERO,
-    &deposit_get_callback,
-    kc);
-  if (NULL == kc->dg)
-  {
-    GNUNET_break (0);
-    destroy_kc (kc);
-  }
-}
-
-
-/**
- * Function called from ``account_kyc_get_status``
- * with KYC status information for this merchant.
- *
- * @param cls a `struct KycContext *`
- * @param h_wire hash of the wire account
- * @param exchange_kyc_serial serial number for the KYC process at the exchange, 0 if unknown
- * @param payto_uri payto:// URI of the merchant's bank account
- * @param exchange_url base URL of the exchange for which this is a status
- * @param last_check when did we last get an update on our KYC status from the exchange
- * @param kyc_ok true if we satisfied the KYC requirements
- * @param aml_decision latest AML decision by the exchange
- */
-static void
-kyc_cb (
-  void *cls,
-  const struct TALER_MerchantWireHashP *h_wire,
-  uint64_t exchange_kyc_serial,
-  const char *payto_uri,
-  const char *exchange_url,
-  struct GNUNET_TIME_Timestamp last_check,
-  bool kyc_ok,
-  enum TALER_AmlDecisionState aml_decision)
-{
-  struct KycContext *kc = cls;
-
-  (void) h_wire;
-  (void) exchange_kyc_serial;
-  (void) payto_uri;
-  (void) exchange_url;
-  kc->kyc_timestamp = last_check;
-  kc->kyc_ok = kyc_ok;
-  /* FIXME: act on aml_decision? */
-}
-
-
-/**
- * Check for our KYC status at @a exchange_url for the
- * payment of @a pc. First checks if we already have a
- * positive result from the exchange, and if not checks
- * with the exchange.
- *
- * @param pc payment context to use as starting point
- * @param eg exchange group of the exchange we are triggering on
- */
-static void
-check_kyc (struct PayContext *pc,
-           const struct ExchangeGroup *eg)
-{
-  enum GNUNET_DB_QueryStatus qs;
-  struct KycContext *kc;
-
-  kc = GNUNET_new (struct KycContext);
-  qs = TMH_db->account_kyc_get_status (TMH_db->cls,
-                                       pc->hc->instance->settings.id,
-                                       &pc->wm->h_wire,
-                                       eg->exchange_url,
-                                       &kyc_cb,
-                                       kc);
-  if (qs < 0)
-  {
-    GNUNET_break (0);
-    GNUNET_free (kc);
-    return;
-  }
-  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
-  {
-    if (kc->kyc_ok)
-    {
-      GNUNET_free (kc);
-      return; /* we are done */
-    }
-    if (GNUNET_TIME_relative_cmp (
-          GNUNET_TIME_absolute_get_duration (
-            kc->kyc_timestamp.abs_time),
-          <,
-          KYC_RETRY_FREQUENCY))
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "Not re-checking KYC status at `%s', as we already recently asked\n",
-                  eg->exchange_url);
-      GNUNET_free (kc);
-      return;
-    }
-  }
-  kc->mi = pc->hc->instance;
-  kc->mi->rc++;
-  kc->wm = pc->wm;
-  kc->exchange_url = GNUNET_strdup (eg->exchange_url);
-  kc->h_contract_terms = pc->h_contract_terms;
-  /* find one of the coins of the batch */
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
-  {
-    struct DepositConfirmation *dc = &pc->dc[i];
-
-    if (0 != strcmp (eg->exchange_url,
-                     pc->dc[i].exchange_url))
-      continue;
-    kc->coin_pub = dc->cdd.coin_pub;
-    break;
-  }
-  GNUNET_CONTAINER_DLL_insert (kc_head,
-                               kc_tail,
-                               kc);
-  kc->fo = TMH_EXCHANGES_keys4exchange (
-    kc->exchange_url,
-    false,
-    &process_kyc_with_exchange,
-    kc);
-  if (NULL == kc->fo)
-  {
-    GNUNET_break (0);
-    destroy_kc (kc);
-  }
-}
-
-
 /**
  * Do database transaction for a completed batch deposit.
  *
@@ -977,7 +651,7 @@ batch_deposit_transaction (const struct ExchangeGroup *eg,
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (pc->amount.currency,
                                         &total_without_fees));
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
     struct TALER_Amount amount_without_fees;
@@ -1013,7 +687,7 @@ batch_deposit_transaction (const struct ExchangeGroup *eg,
   if (qs <= 0)
     return qs; /* Entire batch already known or failure, we're done */
 
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
 
@@ -1120,7 +794,7 @@ handle_batch_deposit_ok (struct ExchangeGroup *eg,
   }
 
   /* Transaction is done, mark affected coins as complete as well. */
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
 
@@ -1132,8 +806,6 @@ handle_batch_deposit_ok (struct ExchangeGroup *eg,
     dc->found_in_db = true;     /* well, at least NOW it'd be true ;-) */
     pc->pending--;
   }
-  check_kyc (pc,
-             eg);
 }
 
 
@@ -1152,11 +824,11 @@ batch_deposit_cb (
   struct PayContext *pc = eg->pc;
 
   eg->bdh = NULL;
+  pc->pending_at_eg--;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Batch deposit completed with status %u\n",
               dr->hr.http_status);
   GNUNET_assert (GNUNET_YES == pc->suspended);
-  pc->pending_at_eg--;
   switch (dr->hr.http_status)
   {
   case MHD_HTTP_OK:
@@ -1255,6 +927,7 @@ process_pay_with_keys (
   unsigned int group_size;
 
   eg->fo = NULL;
+  pc->pending_at_eg--;
   GNUNET_SCHEDULER_begin_async_scope (&hc->async_scope_id);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Processing payment with exchange %s\n",
@@ -1263,7 +936,6 @@ process_pay_with_keys (
   if (NULL == keys)
   {
     GNUNET_break_op (0);
-    pc->pending_at_eg--;
     resume_pay_with_error (
       pc,
       TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT,
@@ -1278,7 +950,6 @@ process_pay_with_keys (
     if (eg->tried_force_keys)
     {
       GNUNET_break_op (0);
-      pc->pending_at_eg--;
       resume_pay_with_error (
         pc,
         TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_WIRE_METHOD_UNSUPPORTED,
@@ -1296,7 +967,6 @@ process_pay_with_keys (
   {
     if (eg->tried_force_keys)
     {
-      pc->pending_at_eg--;
       GNUNET_break_op (0);
       resume_pay_with_error (
         pc,
@@ -1314,7 +984,7 @@ process_pay_with_keys (
   /* Initiate /batch-deposit operation for all coins of
      the current exchange (!) */
   group_size = 0;
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
     const struct TALER_EXCHANGE_DenomPublicKey *denom_details;
@@ -1334,7 +1004,6 @@ process_pay_with_keys (
       if (eg->tried_force_keys)
       {
         GNUNET_break_op (0);
-        pc->pending_at_eg--;
         resume_pay_with_response (
           pc,
           MHD_HTTP_BAD_REQUEST,
@@ -1360,7 +1029,6 @@ process_pay_with_keys (
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Denomination key offered by client has expired for deposits\n");
-      pc->pending_at_eg--;
       resume_pay_with_response (
         pc,
         MHD_HTTP_GONE,
@@ -1409,7 +1077,6 @@ AGE_FAIL:
       if (0 < code)
       {
         GNUNET_break_op (0);
-        pc->pending_at_eg--;
         GNUNET_free (dc->age_commitment.keys);
         resume_pay_with_response (
           pc,
@@ -1434,7 +1101,6 @@ AGE_FAIL:
        * with a coin that has age restriction enabled.  We lack the hash
        * of the age commitment in this case in order to verify the coin
        * and to deposit it with the exchange. */
-      pc->pending_at_eg--;
       GNUNET_break_op (0);
       resume_pay_with_response (
         pc,
@@ -1452,7 +1118,6 @@ AGE_FAIL:
   if (0 == group_size)
   {
     GNUNET_break (0);
-    pc->pending_at_eg--;
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Group size zero, %u batch transactions remain pending\n",
                 pc->pending_at_eg);
@@ -1477,9 +1142,9 @@ AGE_FAIL:
       .refund_deadline = pc->refund_deadline
     };
     enum TALER_ErrorCode ec;
-    unsigned int off = 0;
+    size_t off = 0;
 
-    for (unsigned int i = 0; i<pc->coins_cnt; i++)
+    for (size_t i = 0; i<pc->coins_cnt; i++)
     {
       struct DepositConfirmation *dc = &pc->dc[i];
 
@@ -1509,7 +1174,6 @@ AGE_FAIL:
       /* Signature was invalid or some other constraint was not satisfied.  If
          the exchange was unavailable, we'd get that information in the
          callback. */
-      pc->pending_at_eg--;
       GNUNET_break_op (0);
       resume_pay_with_response (
         pc,
@@ -1520,6 +1184,7 @@ AGE_FAIL:
                                    eg->exchange_url)));
       return;
     }
+    pc->pending_at_eg++;
     if (TMH_force_audit)
       TALER_EXCHANGE_batch_deposit_force_dc (eg->bdh);
   }
@@ -1529,6 +1194,8 @@ AGE_FAIL:
 static void
 force_keys (struct ExchangeGroup *eg)
 {
+  struct PayContext *pc = eg->pc;
+
   eg->tried_force_keys = true;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Forcing /keys download (once) as wire fees are unknown\n");
@@ -1539,15 +1206,13 @@ force_keys (struct ExchangeGroup *eg)
     eg);
   if (NULL == eg->fo)
   {
-    struct PayContext *pc = eg->pc;
-
     GNUNET_break (0);
-    pc->pending_at_eg--;
     resume_pay_with_error (pc,
                            TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_EXCHANGE_LOOKUP_FAILED,
                            "Failed to lookup exchange by URL");
     return;
   }
+  pc->pending_at_eg++;
 }
 
 
@@ -1583,9 +1248,9 @@ get_pay_timeout (unsigned int num_coins)
 {
   struct GNUNET_TIME_Relative t;
 
-  /* FIXME:  Do some benchmarking to come up with a better timeout.
-   * We've increased this value so the wallet integration test passes again
-   * on my (Florian) machine.
+  /* FIXME-Performance-Optimization: Do some benchmarking to come up with a
+   * better timeout.  We've increased this value so the wallet integration
+   * test passes again on my (Florian) machine.
    */
   t = GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS,
                                      15 * (1 + (num_coins / 5)));
@@ -1608,7 +1273,7 @@ phase_batch_deposits (struct PayContext *pc)
     struct ExchangeGroup *eg = pc->egs[i];
     bool have_coins = false;
 
-    for (unsigned int j = 0; j<pc->coins_cnt; j++)
+    for (size_t j = 0; j<pc->coins_cnt; j++)
     {
       struct DepositConfirmation *dc = &pc->dc[j];
 
@@ -1770,7 +1435,7 @@ check_coin_paid (void *cls,
 {
   struct PayContext *pc = cls;
 
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
 
@@ -1844,7 +1509,7 @@ check_coin_refunded (void *cls,
      an abort-pay refund (an unusual but possible case), we need
      to make sure that existing refunds are accounted for. */
 
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
 
@@ -1930,7 +1595,7 @@ check_payment_sufficient (struct PayContext *pc)
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (pc->amount.currency,
                                         &acc_amount));
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
 
@@ -2112,7 +1777,7 @@ check_payment_sufficient (struct PayContext *pc)
       pay_end (pc,
                TALER_MHD_reply_with_error (
                  pc->connection,
-                 MHD_HTTP_NOT_ACCEPTABLE,
+                 MHD_HTTP_BAD_REQUEST,
                  TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_INSUFFICIENT_DUE_TO_FEES,
                  "contract not paid up due to fees (client may have calculated them badly)"));
       return false;
@@ -2121,7 +1786,7 @@ check_payment_sufficient (struct PayContext *pc)
     pay_end (pc,
              TALER_MHD_reply_with_error (
                pc->connection,
-               MHD_HTTP_NOT_ACCEPTABLE,
+               MHD_HTTP_BAD_REQUEST,
                TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_PAYMENT_INSUFFICIENT,
                "payment insufficient"));
     return false;
@@ -2167,7 +1832,7 @@ phase_execute_pay_transaction (struct PayContext *pc)
   GNUNET_break (GNUNET_OK ==
                 TALER_amount_set_zero (pc->amount.currency,
                                        &pc->total_refunded));
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
     pc->dc[i].found_in_db = false;
   pc->pending = pc->coins_cnt;
 
@@ -2367,9 +2032,10 @@ phase_execute_pay_transaction (struct PayContext *pc)
  * @param cls closure with our `struct PayContext *`
  * @param deposit_serial which deposit operation is this about
  * @param exchange_url URL of the exchange that issued the coin
+ * @param h_wire hash of merchant's wire details
+ * @param deposit_timestamp when was the deposit made
  * @param amount_with_fee amount the exchange will deposit for this coin
  * @param deposit_fee fee the exchange will charge for this coin
- * @param h_wire hash of merchant's wire details
  * @param coin_pub public key of the coin
  */
 static void
@@ -2378,13 +2044,14 @@ deposit_paid_check (
   uint64_t deposit_serial,
   const char *exchange_url,
   const struct TALER_MerchantWireHashP *h_wire,
+  struct GNUNET_TIME_Timestamp deposit_timestamp,
   const struct TALER_Amount *amount_with_fee,
   const struct TALER_Amount *deposit_fee,
   const struct TALER_CoinSpendPublicKeyP *coin_pub)
 {
   struct PayContext *pc = cls;
 
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dci = &pc->dc[i];
 
@@ -2435,7 +2102,7 @@ phase_contract_paid (struct PayContext *pc)
                                          "lookup_deposits_by_order"));
     return;
   }
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dci = &pc->dc[i];
 
@@ -2467,7 +2134,7 @@ phase_contract_paid (struct PayContext *pc)
               pc->order_id);
   refunds = json_array ();
   GNUNET_assert (NULL != refunds);
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dci = &pc->dc[i];
     struct TALER_MerchantSignatureP merchant_sig;
@@ -2661,7 +2328,7 @@ phase_check_contract (struct PayContext *pc)
     return;
   }
 
-  for (unsigned int i = 0; i<pc->coins_cnt; i++)
+  for (size_t i = 0; i<pc->coins_cnt; i++)
   {
     struct DepositConfirmation *dc = &pc->dc[i];
 
