@@ -262,6 +262,11 @@ struct GetOrderRequestContext
    */
   struct GNUNET_TIME_Timestamp timestamp;
 
+  /**
+   * Timestamp of the last payment.
+   */
+  struct GNUNET_TIME_Timestamp last_payment;
+
   /**
    * Order summary. Pointer into @e contract_terms.
    */
@@ -996,15 +1001,16 @@ phase_unpaid_finish (struct GetOrderRequestContext *gorc)
  * @param pending true if the this refund was not yet processed by the wallet/exchange
  */
 static void
-process_refunds_cb (void *cls,
-                    uint64_t refund_serial,
-                    struct GNUNET_TIME_Timestamp timestamp,
-                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                    const char *exchange_url,
-                    uint64_t rtransaction_id,
-                    const char *reason,
-                    const struct TALER_Amount *refund_amount,
-                    bool pending)
+process_refunds_cb (
+  void *cls,
+  uint64_t refund_serial,
+  struct GNUNET_TIME_Timestamp timestamp,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  const char *exchange_url,
+  uint64_t rtransaction_id,
+  const char *reason,
+  const struct TALER_Amount *refund_amount,
+  bool pending)
 {
   struct GetOrderRequestContext *gorc = cls;
 
@@ -1013,18 +1019,19 @@ process_refunds_cb (void *cls,
               (unsigned long long) rtransaction_id,
               TALER_amount2s (refund_amount),
               reason);
-  GNUNET_assert (0 ==
-                 json_array_append_new (
-                   gorc->refund_details,
-                   GNUNET_JSON_PACK (
-                     TALER_JSON_pack_amount ("amount",
-                                             refund_amount),
-                     GNUNET_JSON_pack_bool ("pending",
-                                            pending),
-                     GNUNET_JSON_pack_timestamp ("timestamp",
-                                                 timestamp),
-                     GNUNET_JSON_pack_string ("reason",
-                                              reason))));
+  GNUNET_assert (
+    0 ==
+    json_array_append_new (
+      gorc->refund_details,
+      GNUNET_JSON_PACK (
+        TALER_JSON_pack_amount ("amount",
+                                refund_amount),
+        GNUNET_JSON_pack_bool ("pending",
+                               pending),
+        GNUNET_JSON_pack_timestamp ("timestamp",
+                                    timestamp),
+        GNUNET_JSON_pack_string ("reason",
+                                 reason))));
   /* For refunded coins, we are not charged deposit fees, so subtract those
      again */
   for (struct TransferQuery *tq = gorc->tq_head;
@@ -1044,10 +1051,11 @@ process_refunds_cb (void *cls,
         return;
       }
 
-      GNUNET_assert (0 <=
-                     TALER_amount_subtract (&gorc->deposit_fees_total,
-                                            &gorc->deposit_fees_total,
-                                            &tq->deposit_fee));
+      GNUNET_assert (
+        0 <=
+        TALER_amount_subtract (&gorc->deposit_fees_total,
+                               &gorc->deposit_fees_total,
+                               &tq->deposit_fee));
     }
   }
   if (GNUNET_OK !=
@@ -1084,29 +1092,32 @@ phase_check_refunds (struct GetOrderRequestContext *gorc)
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (gorc->contract_amount.currency,
                                         &gorc->refund_amount));
-  qs = TMH_db->lookup_refunds_detailed (TMH_db->cls,
-                                        hc->instance->settings.id,
-                                        &gorc->h_contract_terms,
-                                        &process_refunds_cb,
-                                        gorc);
+  qs = TMH_db->lookup_refunds_detailed (
+    TMH_db->cls,
+    hc->instance->settings.id,
+    &gorc->h_contract_terms,
+    &process_refunds_cb,
+    gorc);
   if (0 > qs)
   {
     GNUNET_break (0);
     phase_end (gorc,
-               TALER_MHD_reply_with_error (gorc->sc.con,
-                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                           TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                           "detailed refunds"));
+               TALER_MHD_reply_with_error (
+                 gorc->sc.con,
+                 MHD_HTTP_INTERNAL_SERVER_ERROR,
+                 TALER_EC_GENERIC_DB_FETCH_FAILED,
+                 "detailed refunds"));
     return;
   }
   if (gorc->refund_currency_mismatch)
   {
     GNUNET_break (0);
     phase_end (gorc,
-               TALER_MHD_reply_with_error (gorc->sc.con,
-                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                           TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                           "refunds in different currency than original order price"));
+               TALER_MHD_reply_with_error (
+                 gorc->sc.con,
+                 MHD_HTTP_INTERNAL_SERVER_ERROR,
+                 TALER_EC_GENERIC_DB_FETCH_FAILED,
+                 "refunds in different currency than original order price"));
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -1127,19 +1138,22 @@ phase_check_refunds (struct GetOrderRequestContext *gorc)
  * @param cls a `struct GetOrderRequestContext`
  * @param deposit_serial identifies the deposit operation
  * @param exchange_url URL of the exchange that issued @a coin_pub
+ * @param h_wire hash of the merchant's wire account into which the deposit was made
+ * @param deposit_timestamp when was the deposit made
  * @param amount_with_fee amount the exchange will deposit for this coin
  * @param deposit_fee fee the exchange will charge for this coin
- * @param h_wire hash of the merchant's wire account into which the deposit was made
  * @param coin_pub public key of the deposited coin
  */
 static void
-deposit_cb (void *cls,
-            uint64_t deposit_serial,
-            const char *exchange_url,
-            const struct TALER_MerchantWireHashP *h_wire,
-            const struct TALER_Amount *amount_with_fee,
-            const struct TALER_Amount *deposit_fee,
-            const struct TALER_CoinSpendPublicKeyP *coin_pub)
+deposit_cb (
+  void *cls,
+  uint64_t deposit_serial,
+  const char *exchange_url,
+  const struct TALER_MerchantWireHashP *h_wire,
+  struct GNUNET_TIME_Timestamp deposit_timestamp,
+  const struct TALER_Amount *amount_with_fee,
+  const struct TALER_Amount *deposit_fee,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub)
 {
   struct GetOrderRequestContext *gorc = cls;
   struct TransferQuery *tq;
@@ -1148,6 +1162,9 @@ deposit_cb (void *cls,
               "Checking deposit status for coin %s (over %s)\n",
               TALER_B2S (coin_pub),
               TALER_amount2s (amount_with_fee));
+  gorc->last_payment
+    = GNUNET_TIME_timestamp_max (gorc->last_payment,
+                                 deposit_timestamp);
   tq = GNUNET_new (struct TransferQuery);
   tq->gorc = gorc;
   tq->exchange_url = GNUNET_strdup (exchange_url);
@@ -1380,7 +1397,12 @@ phase_reply_result (struct GetOrderRequestContext *gorc)
                                  &gorc->claim_token,
                                  h_contract);
   }
-
+  if (GNUNET_TIME_absolute_is_zero (gorc->last_payment.abs_time))
+  {
+    GNUNET_break (GNUNET_YES ==
+                  TALER_amount_is_zero (&gorc->contract_amount));
+    gorc->last_payment = gorc->timestamp;
+  }
   ret = TALER_MHD_REPLY_JSON_PACK (
     gorc->sc.con,
     MHD_HTTP_OK,
@@ -1403,6 +1425,8 @@ phase_reply_result (struct GetOrderRequestContext *gorc)
                                     gorc->contract_terms),
     GNUNET_JSON_pack_string ("order_status",
                              "paid"),
+    GNUNET_JSON_pack_timestamp ("last_payment",
+                                gorc->last_payment),
     GNUNET_JSON_pack_bool ("refunded",
                            gorc->refunded),
     GNUNET_JSON_pack_bool ("wired",
@@ -1443,9 +1467,10 @@ phase_error (struct GetOrderRequestContext *gorc)
 
 
 MHD_RESULT
-TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
-                           struct MHD_Connection *connection,
-                           struct TMH_HandlerContext *hc)
+TMH_private_get_orders_ID (
+  const struct TMH_RequestHandler *rh,
+  struct MHD_Connection *connection,
+  struct TMH_HandlerContext *hc)
 {
   struct GetOrderRequestContext *gorc = hc->ctx;
 
