@@ -370,9 +370,8 @@ credit_cb (
         w->start_row = serial_id;
         return GNUNET_OK;
       }
-      /* FIXME: consider grouping multiple inserts
-         into one bigger transaction with just one
-         notify! */
+      /* FIXME-Performance-Optimization: consider grouping multiple inserts
+         into one bigger transaction with just one notify. */
       credit_payto = TALER_payto_normalize (details->credit_account_uri);
       qs = db_plugin->insert_transfer (db_plugin->cls,
                                        w->instance_id,
