@@ -83,4 +83,25 @@ TALER_MERCHANTDB_token_family_details_free (
 }
 
 
+void
+TALER_MERCHANTDB_category_details_free (
+  struct TALER_MERCHANTDB_CategoryDetails *cd)
+{
+  GNUNET_free (cd->category_name);
+  json_decref (cd->category_name_i18n);
+  for (unsigned int i = 0; i<cd->num_products; i++)
+  {
+    struct TALER_MERCHANTDB_ProductSummary *ps
+      = &cd->products[i];
+
+    GNUNET_free (ps->product_id);
+    GNUNET_free (ps->description);
+    json_decref (ps->description_i18n);
+  }
+  GNUNET_array_grow (cd->products,
+                     cd->num_products,
+                     0);
+}
+
+
 /* end of merchantdb_helper.c */
