@@ -44,6 +44,11 @@
 #include "pg_lookup_instances.h"
 #include "pg_lookup_transfers.h"
 #include "pg_lookup_pending_deposits.h"
+#include "pg_lookup_categories.h"
+#include "pg_select_category.h"
+#include "pg_update_category.h"
+#include "pg_insert_category.h"
+#include "pg_delete_category.h"
 #include "pg_update_wirewatch_progress.h"
 #include "pg_select_wirewatch_accounts.h"
 #include "pg_select_open_transfers.h"
@@ -57,12 +62,14 @@
 #include "pg_account_kyc_set_status.h"
 #include "pg_account_kyc_get_status.h"
 #include "pg_delete_instance_private_key.h"
+#include "pg_select_category_by_name.h"
 #include "pg_purge_instance.h"
 #include "pg_update_instance.h"
 #include "pg_update_instance_auth.h"
 #include "pg_inactivate_account.h"
 #include "pg_activate_account.h"
 #include "pg_lookup_products.h"
+#include "pg_lookup_all_products.h"
 #include "pg_lookup_product.h"
 #include "pg_delete_product.h"
 #include "pg_insert_product.h"
@@ -138,6 +145,8 @@
 #include "pg_lookup_token_families.h"
 #include "pg_delete_token_family.h"
 #include "pg_update_token_family.h"
+#include "pg_insert_token_family_key.h"
+#include "pg_lookup_token_family_key.h"
 
 
 /**
@@ -405,6 +414,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_update_transfer_status;
   plugin->lookup_products
     = &TMH_PG_lookup_products;
+  plugin->lookup_all_products
+    = &TMH_PG_lookup_all_products;
   plugin->lookup_product
     = &TMH_PG_lookup_product;
   plugin->delete_product
@@ -559,6 +570,18 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_insert_pending_webhook;
   plugin->update_pending_webhook
     = &TMH_PG_update_pending_webhook;
+  plugin->lookup_categories
+    = &TMH_PG_lookup_categories;
+  plugin->select_category_by_name
+    = &TMH_PG_select_category_by_name;
+  plugin->select_category
+    = &TMH_PG_select_category;
+  plugin->update_category
+    = &TMH_PG_update_category;
+  plugin->insert_category
+    = &TMH_PG_insert_category;
+  plugin->delete_category
+    = &TMH_PG_delete_category;
   plugin->delete_exchange_accounts
     = &TMH_PG_delete_exchange_accounts;
   plugin->select_accounts_by_exchange
@@ -575,9 +598,14 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_delete_token_family;
   plugin->update_token_family
     = &TMH_PG_update_token_family;
+  plugin->insert_token_family_key
+    = &TMH_PG_insert_token_family_key;
+  plugin->lookup_token_family_key
+    = &TMH_PG_lookup_token_family_key;
   plugin->update_deposit_confirmation_status
     = &TMH_PG_update_deposit_confirmation_status;
 
+
   return plugin;
 }
 
