@@ -27,6 +27,7 @@
 #ifndef TALER_MERCHANT_TESTING_LIB_H
 #define TALER_MERCHANT_TESTING_LIB_H
 
+#include <gnunet/gnunet_time_lib.h>
 #include <taler/taler_testing_lib.h>
 #include "taler_merchant_service.h"
 
@@ -604,6 +605,35 @@ TALER_TESTING_cmd_merchant_post_orders3 (
   const char *amount);
 
 
+/**
+ * Create an order with a choices array with input and output tokens.
+ *
+ * @param label command label
+ * @param cfg configuration to use
+ * @param merchant_url base URL of the merchant serving
+ *        the proposal request.
+ * @param http_status expected HTTP status.
+ * @param token_family_reference label of the POST /tokenfamilies cmd.
+ * @param order_id the name of the order to add.
+ * @param refund_deadline the deadline for refunds on this order.
+ * @param pay_deadline the deadline for payment on this order.
+ * @param amount the amount this order is for.
+ * @return the command
+ */
+
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_orders_choices (
+  const char *label,
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *token_family_reference,
+  const char *order_id,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  struct GNUNET_TIME_Timestamp pay_deadline,
+  const char *amount);
+
+
 /**
  * Define a "GET /orders" CMD.
  *
@@ -1473,6 +1503,40 @@ TALER_TESTING_cmd_merchant_post_using_templates (
   unsigned int http_status);
 
 
+/* ****** Token Families ******* */
+
+
+/**
+ * Define a "POST /tokenfamilies" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *       POST /tokenfamilies request.
+ * @param http_status expected HTTP response code.
+ * @param slug slug of the token family.
+ * @param name name of the token family.
+ * @param description description of the token family.
+ * @param description_i18n internationalized description of the token family.
+ * @param valid_after start of the validity time of the token family.
+ * @param valid_before end of the validity time of the token family.
+ * @param duration validity duration of an issued token of the token family.
+ * @param kind kind of the token family. either "subscription" or "discount".
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_tokenfamilies (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *slug,
+  const char *name,
+  const char *description,
+  json_t *description_i18n,
+  struct GNUNET_TIME_Timestamp valid_after,
+  struct GNUNET_TIME_Timestamp valid_before,
+  struct GNUNET_TIME_Relative duration,
+  const char *kind);
+
 /* ****** Webhooks ******* */
 
 
@@ -1737,7 +1801,10 @@ TALER_TESTING_cmd_checkserver2 (const char *label,
   op (http_method, const char)   \
   op (header_template, const char)   \
   op (body_template, const char)   \
-  op (summary, const char)
+  op (summary, const char)   \
+  op (token_family_slug, const char)   \
+  op (token_family_duration, const struct GNUNET_TIME_Relative)   \
+  op (token_family_kind, const char)
 
 
 /**
